#include <iostream>
#include <cmath>
#include <string>
#include <vector>

using namespace std;

// abstraktni bazova trida uzavrenych oblasti v rovine

class Obrazec {
public:
  Obrazec(std::string n) : nazev_(n) {}
  virtual ~Obrazec();
  virtual double obvod () const = 0;
  virtual double plocha() const = 0;
  std::string nazev() const { return nazev_; }
private:
  std::string nazev_;
};

Obrazec::~Obrazec() {}   // out-of-line virtual function


class Obdelnik : public Obrazec {
public:
  Obdelnik(double a, double b) : Obrazec("obdelnik"), a_(a), b_(b) {}
  ~Obdelnik();
  double obvod ()  const { return 2*(a_ + b_); }
  double plocha()  const { return a_ * b_;     }
  bool   ctverec() const { return std::abs(a_ - b_) > 1e-6;  }
private:
  double a_, b_;
};

Obdelnik::~Obdelnik() {}


class Trojuhelnik : public Obrazec {
public:
  Trojuhelnik(double a, double b, double c) 
  : Obrazec("trojuhelnik"), a_(a), b_(b), c_(c) {}
  ~Trojuhelnik();
  double obvod ()  const { return a_ + b_ + c_; }
  double plocha()  const 
  { 
    double s = obvod()/2;   //Heronuv vzorec 
    return sqrt(s*(s-a_)*(s-b_)*(s-c_));
  }
private:
  double a_, b_, c_;
};

Trojuhelnik::~Trojuhelnik() {}


class Kruh : public Obrazec {
public:
  Kruh(double r) : Obrazec("kruh"),  r_(r) {}
  ~Kruh();
  double obvod ()  const { return 2*M_PI*r_;  }
  double plocha()  const { return M_PI*r_*r_; }
private:
  double r_;
};

Kruh::~Kruh() {}


int main()
{
  // nemuzeme vytvaret objekty typu Obrazec, muzeme ale pracovat s
  // ukazateli na bazovou abstraktni tridu obrazec
  vector<Obrazec*> obrazec;
  obrazec.push_back(new Obdelnik(5,7));
  obrazec.push_back(new Obdelnik(4,4));
  obrazec.push_back(new Trojuhelnik(3,4,5));
  obrazec.push_back(new Kruh(1));
  obrazec.push_back(new Kruh(4));
  for (unsigned int i=0; i<obrazec.size(); i++)
    {
      const Obrazec* n = obrazec[i];
      cout << n->nazev() << "\t" << n->obvod() << " " << n->plocha() << "\n";
    }
}
